package org.zeroturnaround.javarebel.integration.glassfish;

import org.zeroturnaround.bundled.javassist.ClassPool;
import org.zeroturnaround.bundled.javassist.CtClass;
import org.zeroturnaround.bundled.javassist.CtConstructor;
import org.zeroturnaround.javarebel.integration.support.JavassistClassBytecodeProcessor;

/**
 * Transforms <code>org.apache.felix.framework.searchpolicy.ModuleImpl$ModuleClassLoader</code>.
 * 
 * @author Rein Raudjärv
 * 
 * @see FelixModuleImplCBP
 */
public class FelixModuleClassLoaderCBP extends JavassistClassBytecodeProcessor {

  public void process(ClassPool cp, ClassLoader cl, CtClass ctClass) throws Exception {
    cp.importPackage("org.zeroturnaround.javarebel");
    
    /*
     * Registers this class loader with JRebel.
     */
    CtConstructor[] constructors = ctClass.getConstructors();
    for (int i = 0; i < constructors.length; i++) {
      constructors[i].insertAfter(
          "IntegrationFactory.getInstance().registerClassLoader($0, (ClassResourceSource) this$0);");
    }

    /*
     * Delegates class loading the JRebel.
     */
    ctClass.getMethod("findClass", "(Ljava/lang/String;)Ljava/lang/Class;").insertBefore(
        "{ synchronized ($0) {" + 
        "    Class result =" + 
        "      findLoadedClass($1);" + 
        "    if (result != null)" + 
        "      return result;" + 
        "    result = " + 
        "      IntegrationFactory.getInstance().findReloadableClass($0, $1);" + 
        "    if (result != null)" + 
        "      return result;" + 
        "}}");
  }
  
}
