package org.zeroturnaround.javarebel.integration.glassfish;

import org.zeroturnaround.bundled.javassist.CannotCompileException;
import org.zeroturnaround.bundled.javassist.ClassPool;
import org.zeroturnaround.bundled.javassist.CtClass;
import org.zeroturnaround.bundled.javassist.NotFoundException;
import org.zeroturnaround.javarebel.integration.support.JavassistClassBytecodeProcessor;

/**
 * Transforms <code>org.glassfish.web.loader.WebappClassLoader</code>.
 * 
 * @author Rein Raudjärv
 */
public class GlassfishWebappClassLoaderCBP extends JavassistClassBytecodeProcessor {

  public void process(ClassPool cp, ClassLoader cl, CtClass ctClass) throws Exception {
    cp.importPackage("org.zeroturnaround.javarebel");

    processStart(cp, ctClass);
    processReinit(cp, ctClass);
    processFinders(cp, ctClass);
  }

  private void processStart(ClassPool cp, CtClass ctClass) throws CannotCompileException, NotFoundException {
    cp.importPackage("org.zeroturnaround.javarebel.integration.generic");

    ctClass.getDeclaredMethod("start").insertAfter(
        "{ " +
        "  IntegrationFactory.getInstance().registerClassLoader($0, new FindResourceClassResourceSource($0));" +
        "}");
  }

  private void processReinit(ClassPool cp, CtClass ctClass) throws CannotCompileException {
    try {
      ctClass.getDeclaredMethod("addRepository", cp.get(new String[] { "java.net.URL" })).insertBefore(
          "{" +
          "  IntegrationFactory.getInstance().reinitializeClassLoader($0);" +
          "}");
    } catch (NotFoundException e) {
    }
    try {
      ctClass.getDeclaredMethod("addRepository", cp.get(new String[] { "java.lang.String", "java.io.File" })).insertBefore(
          "{" +
          "  IntegrationFactory.getInstance().reinitializeClassLoader($0);" +
          "}");
    } catch (NotFoundException e) {
    }
    try {
      ctClass.getDeclaredMethod("addJar").insertBefore(
          "{" +
          "  IntegrationFactory.getInstance().reinitializeClassLoader($0);" +
          "}");
    } catch (NotFoundException e) {
    }
  }

  private void processFinders(ClassPool cp, CtClass ctClass) throws CannotCompileException, NotFoundException {
    ctClass.getMethod("findClass", "(Ljava/lang/String;)Ljava/lang/Class;").insertBefore(
        "{ synchronized ($0) {" + 
        "    Class result =" + 
        "      findLoadedClass($1);" + 
        "    if (result != null)" + 
        "      return result;" + 
        "    result = " + 
        "      IntegrationFactory.getInstance().findReloadableClass($0, $1);" + 
        "    if (result != null)" + 
        "      return result;" + 
        "}}");

    try {
      ctClass.getDeclaredMethod("findResource", cp.get(new String[] {String.class.getName()})).insertBefore(
          "{" + 
          "  Integration integration = IntegrationFactory.getInstance();" +
          "  if (integration.isResourceReplaced($0, $1))" +
          "    return integration.findResource($0, $1);" + 
          "}");
    }
    catch (NotFoundException e) {
      // skip
    }

    try {
      ctClass.getDeclaredMethod("findResources", cp.get(new String[] {String.class.getName()})).insertBefore(
          "{" + 
          "  Integration integration = IntegrationFactory.getInstance();" +
          "  if (integration.isResourceReplaced($0, $1))" +
          "    return integration.findResources($0, $1);" + 
          "}");
    }
    catch (NotFoundException e) {
      // skip
    }

    ctClass.getDeclaredMethod("findLoadedResource").setBody(
        "{ " +
        "  java.net.URL url = findResource($1);" +
        "  try {" +
        "    return url != null ? url.openStream() : null;" +
        "  } catch (java.io.IOException e) {" +
        "    return null;" +
        "  }" +
        "}");
  }
  

}
