package org.zeroturnaround.javarebel.integration.google;

import org.zeroturnaround.bundled.javassist.CannotCompileException;
import org.zeroturnaround.bundled.javassist.ClassPool;
import org.zeroturnaround.bundled.javassist.CtClass;
import org.zeroturnaround.bundled.javassist.CtConstructor;
import org.zeroturnaround.bundled.javassist.CtNewMethod;
import org.zeroturnaround.bundled.javassist.NotFoundException;
import org.zeroturnaround.javarebel.integration.support.JavassistClassBytecodeProcessor;

/**
 * Processes <code>com.google.gwt.dev.shell.CompilingClassLoader</code>
 * to reload client side classes in GWT hosted mode.
 * 
 * @author Rein Raudjärv
 */
public class GwtCompilingClassLoaderCBP extends JavassistClassBytecodeProcessor {

  public void process(ClassPool cp, ClassLoader cl, CtClass ctClass) throws Exception {
    cp.importPackage("org.zeroturnaround.javarebel");
    
    implementClassResourceSource(cp, ctClass);
    
    /*
     * Register ClassLoader.
     */
    CtConstructor[] constructors = ctClass.getConstructors();
    for (int i = 0; i < constructors.length; i++) {
      constructors[i].insertAfter(
          "IntegrationFactory.getInstance().registerClassLoader($0, (ClassResourceSource) $0);");
    }

    /*
     * Override class loading.
     */
    ctClass.getDeclaredMethod("findClass").insertBefore(
        "{ synchronized ($0) {" + 
        "    Class result =" + 
        "      findLoadedClass($1);" + 
        "    if (result != null)" + 
        "      return result;" + 
        "    result = " + 
        "      IntegrationFactory.getInstance().findReloadableClass($0, $1);" + 
        "    if (result != null)" + 
        "      return result;" + 
        "}}");
  }

  private void implementClassResourceSource(ClassPool cp, CtClass ctClass) throws CannotCompileException, NotFoundException {
    cp.importPackage("java.net");
    cp.importPackage("org.zeroturnaround.javarebel.integration.util");

    ctClass.addInterface(cp.get("org.zeroturnaround.javarebel.ClassResourceSource"));
    
    /*
     * The context ClassLoader is used to get the class resource.
     * 
     * See com.google.gwt.dev.javac.CompilationUnit.getClassBytes().
     */
    ctClass.addMethod(CtNewMethod.make(
        "public Resource getClassResource(String className) {" +
        "  try {" +
        "    ClassLoader cl = Thread.currentThread().getContextClassLoader();" +
        "    String name = className.replace('.', '/') + \".class\";" +
        "    URL url = cl.getResource(name);" +
        "    if (url != null)" +
        "      return ResourceUtil.asResource(url);" +
        "  } catch (Throwable t) {" +
        "    LoggerFactory.getInstance().error(t);" +
        "  }" +
        "  return null;" +
        "}",
        ctClass));
    
    /*
     * ClassLoader API is not available in client side code.
     */
    ctClass.addMethod(CtNewMethod.make(
        "public Resource getLocalResource(String name) { return null; }", ctClass));
    ctClass.addMethod(CtNewMethod.make(
        "public Resource[] getLocalResources(String name) { return null; }", ctClass));
  }
  
}
