package org.zeroturnaround.javarebel.integration.streambase;

import org.zeroturnaround.bundled.javassist.CannotCompileException;
import org.zeroturnaround.bundled.javassist.ClassPool;
import org.zeroturnaround.bundled.javassist.CtClass;
import org.zeroturnaround.bundled.javassist.CtNewMethod;
import org.zeroturnaround.bundled.javassist.NotFoundException;
import org.zeroturnaround.javarebel.LoggerFactory;
import org.zeroturnaround.javarebel.integration.generic.FindClassClassBytecodeProcessor;

/**
 * Transforms <code>com.streambase.sb.runtime.JarClassLoader</code>.
 * 
 * @author Rein Raudjärv
 */
public class StreamBaseJarClassLoaderCBP extends FindClassClassBytecodeProcessor {

  public void process(ClassPool cp, ClassLoader cl, CtClass ctClass) throws Exception {
    super.process(cp, cl, ctClass);
    reinitializeClassLoader(cp, ctClass);
    resourceProviding(cp, ctClass);
  }

  private void reinitializeClassLoader(ClassPool cp, CtClass ctClass) throws CannotCompileException {
    cp.importPackage("org.zeroturnaround.javarebel");

    /*
     * Reinitialize ClassLoader if new JAR is added or an existing is removed
     */
    try {
      ctClass.getDeclaredMethod("addJar").insertAfter(
          "{" +
          "  IntegrationFactory.getInstance().reinitializeClassLoader($0);" +
      "}");
    } catch (NotFoundException e) {
      LoggerFactory.getInstance().error(e);
    }
    try {
      ctClass.getDeclaredMethod("removeJar").insertAfter(
          "{" +
          "  IntegrationFactory.getInstance().reinitializeClassLoader($0);" +
          "}");
    } catch (NotFoundException e) {
      LoggerFactory.getInstance().error(e);
    }
  }

  private void resourceProviding(ClassPool cp, CtClass ctClass) throws CannotCompileException {
    cp.importPackage("java.io");
    cp.importPackage("java.net");
    cp.importPackage("java.util");
    cp.importPackage("java.util.jar");
    cp.importPackage("org.zeroturnaround.javarebel");
    cp.importPackage("org.zeroturnaround.javarebel.integration.streambase");
    cp.importPackage("com.streambase.sb.util");
    
    boolean findResourceMissing = false;
    try {
      ctClass.getDeclaredMethod("findResource");
    } catch (NotFoundException e) {
      findResourceMissing = true;
    }
    
    if (findResourceMissing) {
      try {
        ctClass.removeMethod(ctClass.getDeclaredMethod("getResourceAsStream"));
      } catch (NotFoundException e) {
        LoggerFactory.getInstance().error(e);
      }
      
      ctClass.addMethod(CtNewMethod.make(
          "protected URL findResource(String name) {" +
          
          "  Integration integration = IntegrationFactory.getInstance();" +
          "  if (integration.isResourceReplaced($0, $1))" +
          "    return integration.findResource($0, $1);" +
          
          "  Msg.debug2(\"find findResource: {0}\", new Object[] { name });" +
          
          "  Iterator i = this.jars.values().iterator();" +
          "  if (i.hasNext()) {" +
          "    JarFile jf = (JarFile)i.next();" +
          "    JarEntry je = jf.getJarEntry(name);" +
          "    if (je != null) {" +
          "      return JarUtil.toURL(jf, je);" +
          "    }" +
          "  }" +
          
          "  return null;" +
          
          "}", ctClass));
      
      System.out.println("aaaaa");
    }
  }
  
}
